﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PublishSubscribeInterfaces
{
    class Program
    {
        static void Main(string[] args)
        {
            MagazinePublisher mp = new MagazinePublisher();
            Subscriber s1 = new Subscriber();
            Subscriber s2 = new Subscriber();
            
           var removeMe1 = mp.Subscribe(s1);
           var removeMe2 = mp.Subscribe(s2);

            // 'setting' this NewIssue property - will call notify on all the subscribers
            mp.NewIssue = DateTime.Now.ToString("MMMM") + " issue";
            
            // Cleanup 
            removeMe1.Dispose();
            removeMe2.Dispose();
        }
    }

    /// <summary>
    ///  'Real-World' publisher - inherits from our implementation Publisher. Specify the type to be MagazinePublisher as well
    /// </summary>
    class MagazinePublisher : Publisher<MagazinePublisher>
    {
        string _newIssue;

        public string NewIssue
        {
            get { return _newIssue; }
            set { 
                _newIssue = value;
                Notify(this);
            }
        }
    }

    /// <summary>
    /// 'Real-World' Subscriber - Notice that it takes the type parameter MagazinePublisher in the IObserver interface
    /// </summary>
    class Subscriber : IObserver<MagazinePublisher>
    {
        public void OnCompleted()
        {
            Console.WriteLine("Unsubscribed");
        }

        public void OnError(Exception error)
        {
            Console.WriteLine("Error");
        }

        public void OnNext(MagazinePublisher mp)
        {
            Console.WriteLine("New Issue: " + mp.NewIssue);
        }
    }
}
